package org.hardknockjava.lesson03.excercise01;

/**
 * Temperature is an abstract class representing a temperature
 * in either Fahrenheit, Celsius, or Kelvin.
 * @see TemperatureCelsius
 * @see TemperatureFahrenheit
 * @see TemperatureKelvin
 * @author Hard Knock Java
 */
public abstract class Temperature {
    /**
     * The word "Fahrenheit"
     */
	public static final String SCALE_FAHRENHEIT = "Fahrenheit";
    /**
     * The word "Celsius"
     */
	public static final String SCALE_CELSIUS = "Celsius";
    /**
     * The word "Kelvin"
     */
	public static final String SCALE_KELVIN = "Kelvin";

	/** degrees is the temperature in Fahrenheit or in Celsius,
	*   depending on the concrete class.
	*/
	protected double degrees;
	
	/**
	 * CELSIUS_TO_KELVIN is the value added to a Celsius
	 * degree reading to convert to Kelvin--or to subtract
	 * from Kelvin to get Celsius.
	 */
	protected static final double CELSIUS_TO_KELVIN = 273.15;

	/**
	 * Superclass constructor stores the value of degrees.
	 */
	protected Temperature(double degrees) {
		super();
		this.degrees = degrees;
	}
	
	/*
	 *  Abstract methods to be implemented in subclasses.
	 */
	/**
	 * Return the temperature in degrees Fahrenheit.
	 * @return The temperature in degrees Fahrenheit.
	 */
	public abstract double getFahrenheit();
	
	/**
	 * Return the temperature in degrees Celsius.
	 * @return The temperature in degrees Celsius.
	 */
	public abstract double getCelsius();
	
	// getKelvin works the same for everybody--get the Celsius value
	// and add CELSIUS_TO_KELVIN--so there's no need to override it
	// in the subclasses.
	/**
	 * Return the temperature in degrees Kelvin.
	 * @return The temperature in degrees Kelvin.
	 */
	public double getKelvin() {
		return getCelsius() + CELSIUS_TO_KELVIN;
	}
	
	// Return the raw degree value and the scale of this temperature.
	// We're adding these so the driver can display them.
	/**
	 * Return the temperature value passed to the constructor in degrees, 
	 * as supplied when the object was created.
	 * @return The raw temperature in degrees.
	 */
	public double getDegrees() {
		return degrees;
	}
	
	/**
	 * Return the temperature scale used by the object.
	 * @return The temperature scale used.
	 */
	public abstract String getScale();
}
